@@ -1,8 +1,14 @@
 from collections import Counter
+from typing import Dict, Optional, Tuple, Type, TypeVar
+
+from django.db import models
 from django.db.models import query
 
+from .config import HARD_DELETE, NO_DELETE
 from .query import SafeDeleteQuery
 
+_QS = TypeVar('_QS', bound='SafeDeleteQueryset')
+
 
 class SafeDeleteQueryset(query.QuerySet):
     """Default queryset for the SafeDeleteManager.
@@ -13,11 +19,17 @@ class SafeDeleteQueryset(query.QuerySet):
     QuerySet itself is evaluated.
     """
 
-    def __init__(self, model=None, query=None, using=None, hints=None):
+    def __init__(
+            self,
+            model: Optional[Type[models.Model]] = None,
+            query: Optional[SafeDeleteQuery] = None,
+            using: Optional[str] = None,
+            hints: Optional[Dict[str, models.Model]] = None
+    ):
         super(SafeDeleteQueryset, self).__init__(model=model, query=query, using=using, hints=hints)
-        self.query = query or SafeDeleteQuery(self.model)
+        self.query: SafeDeleteQuery = query or SafeDeleteQuery(self.model)
 
-    def delete(self, force_policy=None):
+    def delete(self, force_policy: Optional[int] = None) -> Tuple[int, Dict[str, int]]:
         """Overrides bulk delete behaviour.
 
         .. note::
@@ -28,16 +40,27 @@ class SafeDeleteQueryset(query.QuerySet):
             :py:func:`safedelete.models.SafeDeleteModel.delete`
         """
         assert self.query.can_filter(), "Cannot use 'limit' or 'offset' with delete."
-        deleted_counter = Counter()
-        # TODO: Replace this by bulk update if we can
-        for obj in self.all():
-            _, delete_response = obj.delete(force_policy=force_policy)
-            deleted_counter.update(delete_response)
-        self._result_cache = None
-        return sum(deleted_counter.values()), dict(deleted_counter)
-    delete.alters_data = True
 
-    def undelete(self, force_policy=None):
+        if force_policy == NO_DELETE:
+            return (0, {})
+        elif force_policy == HARD_DELETE:
+            return self.hard_delete_policy_action()
+        else:
+            deleted_counter: Counter = Counter()
+            # TODO: Replace this by bulk update if we can
+            for obj in self.all():
+                _, delete_response = obj.delete(force_policy=force_policy)
+                deleted_counter.update(delete_response)
+            self._result_cache = None
+            return sum(deleted_counter.values()), dict(deleted_counter)
+    delete.alters_data = True  # type: ignore
+
+    def hard_delete_policy_action(self) -> Tuple[int, Dict[str, int]]:
+        # Normally hard-delete the objects.
+        self.query._filter_visibility()
+        return super().delete()
+
+    def undelete(self, force_policy: Optional[int] = None) -> Tuple[int, Dict[str, int]]:
         """Undelete all soft deleted models.
 
         .. note::
@@ -48,16 +71,16 @@ class SafeDeleteQueryset(query.QuerySet):
             :py:func:`safedelete.models.SafeDeleteModel.undelete`
         """
         assert self.query.can_filter(), "Cannot use 'limit' or 'offset' with undelete."
-        undeleted_counter = Counter()
+        undeleted_counter: Counter = Counter()
         # TODO: Replace this by bulk update if we can (need to call pre/post-save signal)
         for obj in self.all():
             _, undelete_response = obj.undelete(force_policy=force_policy)
             undeleted_counter.update(undelete_response)
         self._result_cache = None
         return sum(undeleted_counter.values()), dict(undeleted_counter)
-    undelete.alters_data = True
+    undelete.alters_data = True  # type: ignore
 
-    def all(self, force_visibility=None):
+    def all(self: _QS, force_visibility=None) -> _QS:
         """Override so related managers can also see the deleted models.
 
         A model's m2m field does not easily have access to `all_objects` and
