@@ -8,7 +8,7 @@ from django.contrib.auth.models import User
 from django.db import models
 from django.test import RequestFactory, TestCase
 
-from ..admin import SafeDeleteAdmin, highlight_deleted
+from ..admin import SafeDeleteAdmin, SafeDeleteAdminFilter, highlight_deleted
 from ..config import FIELD_NAME
 from ..models import SafeDeleteModel
 from .models import Article, Author, Category
@@ -78,6 +78,9 @@ class AdminTestCase(TestCase):
         # New parameter in Django 2.1
         if hasattr(modeladmin, 'sortable_by'):
             args.append(modeladmin.sortable_by)
+        # New parameter in Django 4.0
+        if hasattr(modeladmin, 'search_help_text'):
+            args.append(modeladmin.search_help_text)
         return ChangeList(*args)
 
     def test_admin_model(self):
@@ -87,12 +90,44 @@ class AdminTestCase(TestCase):
         self.assertEqual(changelist.queryset.count(), 3)
         self.assertEqual(changelist_default.queryset.count(), 2)
 
-    def test_admin_listing(self):
+    def test_highlight_deleted(self):
         """Test deleted objects are in red in admin listing."""
         resp = self.client.get('/admin/safedelete/category/')
         line = '<span class="deleted">{0}</span>'.format(self.categories[1])
         self.assertContains(resp, line)
 
+    def test_highlight_deleted_field(self):
+        self.modeladmin.field_to_highlight = "name"
+        self.modeladmin.list_display = ("highlight_deleted_field",)
+        resp = self.client.get('/admin/safedelete/category/')
+        line = '<span class="deleted">{0}</span>'.format(self.categories[1])
+        self.assertContains(resp, line)
+
+    def test_soft_delete_admin_filter(self):
+        self.modeladmin.list_filter += (SafeDeleteAdminFilter,)
+
+        with self.subTest("test_filter_unfiltered"):
+            changelist = self.get_changelist(self.request, Category, self.modeladmin)
+            queryset = changelist.get_queryset(self.request)
+            self.assertIn(Category.all_objects.get(pk=self.categories[0].pk), queryset)
+            self.assertNotIn(Category.all_objects.get(pk=self.categories[1].pk), queryset)
+
+        with self.subTest("test_filter_equals_FIELD_NAME"):
+            request = self.request_factory.get('/', {FIELD_NAME: FIELD_NAME})
+            request.user = self.request.user
+            changelist = self.modeladmin.get_changelist_instance(request)
+            queryset = changelist.get_queryset(request)
+            self.assertIn(Category.all_objects.get(pk=self.categories[0].pk), queryset)
+            self.assertIn(Category.all_objects.get(pk=self.categories[1].pk), queryset)
+
+        with self.subTest("test_filter_equals_FIELD_NAME_only"):
+            request = self.request_factory.get('/', {FIELD_NAME: FIELD_NAME + '_only'})
+            request.user = self.request.user
+            changelist = self.modeladmin.get_changelist_instance(request)
+            queryset = changelist.get_queryset(request)
+            self.assertNotIn(Category.all_objects.get(pk=self.categories[0].pk), queryset)
+            self.assertIn(Category.all_objects.get(pk=self.categories[1].pk), queryset)
+
     def test_admin_xss(self):
         """Test whether admin XSS is blocked."""
         Category.objects.create(name='<script>alert(42)</script>'),
