@@ -5,8 +5,8 @@ from django.contrib.contenttypes.fields import (
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import FieldError
 from django.db import models
+from django.db.models import ProtectedError
 from django.test import TestCase
-from django.core.exceptions import FieldError
 
 from safedelete import SOFT_DELETE, SOFT_DELETE_CASCADE
 from safedelete.config import DELETED_BY_CASCADE_FIELD_NAME
@@ -191,6 +191,12 @@ class SimpleTest(TestCase):
 
         self.assertEqual(PressNormalModel.objects.first().article, self.articles[0])
 
+    def test_soft_delete_cascade_with_protect(self):
+        PressNormalModel.article.field.remote_field.on_delete = models.PROTECT
+        PressNormalModel.objects.create(name='press 0', article=self.articles[2])
+        with self.assertRaises(ProtectedError):
+            self.authors[2].delete(force_policy=SOFT_DELETE_CASCADE)
+
     def test_soft_delete_cascade_with_abstract_model(self):
         ArticleView.objects.create(article=self.articles[2])
 
